#!/usr/bin/env python
# -*- coding: utf8 -*-

#-----------------------------------------
# This is a part of Collège 3D project.
# Author:       Pascal Peter
# Copyright:    (C) 2003-2016 Pascal Peter
# Licence:      GNU General Public Licence version 3
# Website:      http://pascal.peter.free.fr/
# Email:        pascal.peter at free.fr
#-----------------------------------------


################################################################################
# Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le
# modifier conformément aux dispositions de la Licence Publique Générale GNU,
# telle que publiée par la Free Software Foundation ; version 3 de la licence,
# ou encore toute version ultérieure.
# 
# Ce programme est distribué dans l'espoir qu'il sera utile,
# mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de COMMERCIALISATION
# ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de détail,
# voir la Licence Publique Générale GNU.
# 
# Vous devez avoir reçu un exemplaire de la Licence Publique Générale GNU en même
# temps que ce programme ; si ce n'est pas le cas, voir <http://www.gnu.org/licenses/>.
# -------------------------------------------------------------------------------
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
################################################################################



"""
DESCRIPTION :
    Lance le collège 3D.
"""



# **************************************************
#       IMPORTATION DES MODULES UTILES
# **************************************************

from __future__ import print_function
import sys
import os
import subprocess



# **************************************************
#       POUR FONCTIONNER AVEC PYTHON 2.x OU 3.x
# **************************************************
PYTHONVERSION = sys.version_info[0] * 10 + sys.version_info[1]

def myPrint(*args):
    if len(args) > 1:
        print(args)
    else:
        arg = args[0]
        try:
            print(arg)
        except:
            try:
                print(u(arg))
            except:
                try:
                    print(s(arg))
                except:
                    print('PB in myPrint')

def u(text):
    # retourne une version unicode de text
    if PYTHONVERSION >= 30:
        try:
            if isinstance(text, str):
                return text
            else:
                return str(text)
        except:
            myPrint('ERROR utils.u', type(text), text)
            return text
    else:
        try:
            return unicode(text)
        except:
            if isinstance(text, str):
                return text.decode('utf-8')
            else:
                myPrint('ERROR utils.u', type(text), text)
                return text



# **************************************************
#       VARIABLES UTILES
# **************************************************
DEBUG = False
if 'DEBUG' in sys.argv:
    DEBUG = True

HERE = os.path.dirname(os.path.abspath(__file__))
if DEBUG:
    myPrint('HERE:')
    myPrint(HERE)

SHORT_NAME = 'college3d'
EXECUTABLE = ''



# **************************************************
#       DÉTECTION DU SYSTÈME
# **************************************************
def detectPlatform():
    global EXECUTABLE
    result = False
    dirName = ''
    osDir = ''
    try:
        dirName = HERE.split(os.sep)[-1]
    except:
        pass
    if dirName == 'college3d':
        # 32 ou 64 bits :
        if sys.maxsize > 2**32:
            bits = 64
        else:
            bits = 32
        # osName :
        osName = ''
        if sys.platform.startswith('linux'):
            osName = 'linux'
            EXECUTABLE = 'college3d'
        elif sys.platform.startswith('win'):
            osName = 'win'
            EXECUTABLE = 'college3d.exe'
        else:
            osName = 'mac'
            EXECUTABLE = 'college3d.app'
        # on cherche le fichier executable (dans bin) :
        dirsToTest = ('{0}{1}'.format(osName, bits), '{0}32'.format(osName), osName)
        for dirToTest in dirsToTest:
            expandedDir = '{1}{0}bin{0}college3d_{2}'.format(os.sep, HERE, dirToTest)
            if (osDir == '') and (os.path.isdir(expandedDir)):
                osDir = expandedDir
    else:
        if sys.platform.startswith('linux'):
            EXECUTABLE = 'college3d'
        elif sys.platform.startswith('win'):
            EXECUTABLE = 'college3d.exe'
        else:
            EXECUTABLE = 'college3d.app'
        osDir = HERE
    if osDir != '':
        """
        if osName in ('powerpc', 'mac'):
            # il faut encore trouver le bon sous-dossier :
            BLENDER_PLAYER = 'blenderplayer.app/Contents/MacOS/blenderplayer'
        """
        EXECUTABLE = '{1}{0}{2}'.format(os.sep, osDir, EXECUTABLE)
        result = True
    else:
        myPrint('THIS PLATFORM IS NOT SUPPORTED : {0} {1}'.format(osName, bits))
    return result



# **************************************************
#       LECTURE DE LA CONFIG
# **************************************************
def findConfigDir():
    configDir = ''
    if sys.platform.startswith('win'):
        configDir = os.environ['APPDATA']
        configDir = os.path.join(configDir, SHORT_NAME)
    else:
        configsDir = os.path.join(os.path.expanduser('~'), '.config')
        if os.path.isdir(configsDir):
            configDir = os.path.join(configsDir, SHORT_NAME)
        else:
            configDir = os.path.join(os.path.expanduser('~'), '.{0}'.format(SHORT_NAME))
    # on vérifie l'existence du dossier :
    if not(os.path.isdir(configDir)):
        os.mkdir(configDir)
    if not(os.path.isdir(configDir)):
        configDir = '{1}{0}game'.format(os.sep, HERE)
    return configDir

def loadSettings():
    if DEBUG:
        myPrint('loadSettings in College3D.pyw')
    globalDict = {}
    configDir = findConfigDir()
    configFile = os.path.join(configDir, 'college3d.json')
    if os.path.isfile(configFile):
        import json
        try:
            if PYTHONVERSION >= 30:
                theFile = open(configFile, newline='', encoding='utf-8')
            else:
                theFile = open(configFile, 'rb')
            globalDict = json.load(theFile)
            theFile.close()
        except:
            globalDict = {}
    else:
        globalDict = {}
    fullScreen, anaglyphs = False, False
    if 'MENU' in globalDict:
        if 'fullScreen' in globalDict['MENU']:
            fullScreen = globalDict['MENU']['fullScreen']
        if 'anaglyphs' in globalDict['MENU']:
            anaglyphs = globalDict['MENU']['anaglyphs']
    return (fullScreen, anaglyphs)



# **************************************************
#       CRÉATION DE LA LIGNE DE COMMANDE
#       ET LANCEMENT
# **************************************************
if detectPlatform():
    (fullScreen, anaglyphs) = loadSettings()
    # paramètres :
    paramsBP = ''
    if 'SMALL' in sys.argv:
        coeff = 3
        width = 160 * coeff
        height = 120 * coeff
        paramsBP = '{0} -w {1} {2}'.format(paramsBP, width, height)
    elif fullScreen:
        paramsBP = '{0} -f'.format(paramsBP)
    if anaglyphs:
        paramsBP = '{0} -s anaglyph'.format(paramsBP)
    if 'VIDEO' in sys.argv:
        paramsBP = '{0} VIDEO'.format(paramsBP)
    paramsBP = paramsBP.split()
    # la liste des arguments du lancement :
    args = [EXECUTABLE]
    args.extend(paramsBP)
    if DEBUG:
        myPrint('args :')
        myPrint(args)
    # on peut lancer le jeu :
    try:
        process = subprocess.Popen(args)
    except:
        myPrint('EXECUTION FAILED')
else:
    # chemin vers le EXECUTABLE :
    myPrint('EXECUTION FAILED:')
    myPrint(EXECUTABLE)
