#!/usr/bin/env python
# -*- coding: utf8 -*-

#-----------------------------------------
# This is a part of Collège 3D project.
# Author:       Pascal Peter
# Copyright:    (C) 2003-2016 Pascal Peter
# Licence:      GNU General Public Licence version 3
# Website:      http://pascal.peter.free.fr/
# Email:        pascal.peter at free.fr
#-----------------------------------------

"""
DESCRIPTION :
    programme serveur pour un jeu en réseau
    d'après Swinnen : http://inforef.be/swi/python.htm
"""

# importation des modules utiles :
from __future__ import print_function

import sys
import socket
#import threading
#import time



# **************************************************
#       POUR FONCTIONNER AVEC PYTHON 2.x OU 3.x
# **************************************************
PYTHONVERSION = sys.version_info[0] * 10 + sys.version_info[1]

if PYTHONVERSION >= 30:
    import asyncoro3 as asyncoro
else:
    import asyncoro2 as asyncoro



def toSend(msg):
    if PYTHONVERSION >= 30:
        return bytes(msg, 'utf8')
    return msg



# **************************************************
#       RÉCUPÉRATION DE L'IP DU POSTE
# **************************************************
def findMyIp():
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(('google.com', 80))
        myIp = s.getsockname()[0]
        s.close()
    except:
        myIp = 'localhost'
    return myIp



# **************************************************
#       PARAMÈTRES DE CONNEXION
# **************************************************
HOST = ''
for arg in sys.argv:
    if arg.split('=')[0] == 'HOST':
        HOST = arg.split('=')[1]
if HOST == '':
    HOST = findMyIp()
LAN_PORT = 5005
BUFFER = 1024










def client_proc(conn, addr, server, coro=None):
    # given conn is synchronous, convert it to asynchronous
    conn = asyncoro.AsyncSocket(conn)
    server.send(('joined', (conn, addr)))

    while True:
        line = yield conn.recv_msg()
        if not line:
            server.send(('left', (conn, addr)))
            break
        server.send(('broadcast', (conn, line)))

def server_proc(coro=None):
    clients = set()
    while True:
        cmd, item = yield coro.receive()
        if cmd == 'broadcast':
            conn, msg = item
            if PYTHONVERSION >= 30:
                msg = msg.decode('utf-8')
            msg = '{0} says: {1}'.format(conn.fileno(), msg)
            msg = toSend(msg)
            print(clients)
            for client in clients:
                if conn != client:
                    yield client.send_msg(msg)
        elif cmd == 'joined':
            conn, addr = item
            msg = 'id: {0}'.format(conn.fileno())
            msg = toSend(msg)
            yield conn.send_msg(msg)
            msg = '{0} joined'.format(conn.fileno())
            msg = toSend(msg)
            for client in clients:
                yield client.send_msg(msg)
            clients.add(conn)
        elif cmd == 'left':
            conn, addr = item
            clients.discard(conn)
            msg = '{0} left'.format(conn.fileno())
            msg = toSend(msg)
            conn.close()
            for client in clients:
                yield client.send_msg(msg)
        elif cmd == 'terminate':
            print('SERVER TERMINATE 2')
            break



# **************************************************
#       PROGRAMME PRINCIPAL
# **************************************************
def main():
    mySocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    mySocket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    try:
        mySocket.bind((HOST, LAN_PORT))
    except socket.error:
        print('SERVER SOCKET ERROR')
        return False
    print('SERVER READY ...')
    mySocket.listen(128)

    #asyncoro.logger.info('server at %s', str(mySocket.getsockname()))
    server = asyncoro.Coro(server_proc)
    while True:
        conn, addr = mySocket.accept()
        # each connection is handled by a coroutine
        asyncoro.Coro(client_proc, conn, addr, server)
    print('SERVER TERMINATE')
    server.send(('terminate', None))


if __name__ == "__main__":
    main()
